/*
Copyright The Volcano Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
// Code generated by client-gen. DO NOT EDIT.

package fake

import (
	"context"
	json "encoding/json"
	"fmt"

	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	labels "k8s.io/apimachinery/pkg/labels"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	testing "k8s.io/client-go/testing"
	v1alpha1 "volcano.sh/apis/pkg/apis/topology/v1alpha1"
	topologyv1alpha1 "volcano.sh/apis/pkg/client/applyconfiguration/topology/v1alpha1"
)

// FakeHyperNodes implements HyperNodeInterface
type FakeHyperNodes struct {
	Fake *FakeTopologyV1alpha1
}

var hypernodesResource = v1alpha1.SchemeGroupVersion.WithResource("hypernodes")

var hypernodesKind = v1alpha1.SchemeGroupVersion.WithKind("HyperNode")

// Get takes name of the hyperNode, and returns the corresponding hyperNode object, and an error if there is any.
func (c *FakeHyperNodes) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1alpha1.HyperNode, err error) {
	emptyResult := &v1alpha1.HyperNode{}
	obj, err := c.Fake.
		Invokes(testing.NewRootGetActionWithOptions(hypernodesResource, name, options), emptyResult)
	if obj == nil {
		return emptyResult, err
	}
	return obj.(*v1alpha1.HyperNode), err
}

// List takes label and field selectors, and returns the list of HyperNodes that match those selectors.
func (c *FakeHyperNodes) List(ctx context.Context, opts v1.ListOptions) (result *v1alpha1.HyperNodeList, err error) {
	emptyResult := &v1alpha1.HyperNodeList{}
	obj, err := c.Fake.
		Invokes(testing.NewRootListActionWithOptions(hypernodesResource, hypernodesKind, opts), emptyResult)
	if obj == nil {
		return emptyResult, err
	}

	label, _, _ := testing.ExtractFromListOptions(opts)
	if label == nil {
		label = labels.Everything()
	}
	list := &v1alpha1.HyperNodeList{ListMeta: obj.(*v1alpha1.HyperNodeList).ListMeta}
	for _, item := range obj.(*v1alpha1.HyperNodeList).Items {
		if label.Matches(labels.Set(item.Labels)) {
			list.Items = append(list.Items, item)
		}
	}
	return list, err
}

// Watch returns a watch.Interface that watches the requested hyperNodes.
func (c *FakeHyperNodes) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	return c.Fake.
		InvokesWatch(testing.NewRootWatchActionWithOptions(hypernodesResource, opts))
}

// Create takes the representation of a hyperNode and creates it.  Returns the server's representation of the hyperNode, and an error, if there is any.
func (c *FakeHyperNodes) Create(ctx context.Context, hyperNode *v1alpha1.HyperNode, opts v1.CreateOptions) (result *v1alpha1.HyperNode, err error) {
	emptyResult := &v1alpha1.HyperNode{}
	obj, err := c.Fake.
		Invokes(testing.NewRootCreateActionWithOptions(hypernodesResource, hyperNode, opts), emptyResult)
	if obj == nil {
		return emptyResult, err
	}
	return obj.(*v1alpha1.HyperNode), err
}

// Update takes the representation of a hyperNode and updates it. Returns the server's representation of the hyperNode, and an error, if there is any.
func (c *FakeHyperNodes) Update(ctx context.Context, hyperNode *v1alpha1.HyperNode, opts v1.UpdateOptions) (result *v1alpha1.HyperNode, err error) {
	emptyResult := &v1alpha1.HyperNode{}
	obj, err := c.Fake.
		Invokes(testing.NewRootUpdateActionWithOptions(hypernodesResource, hyperNode, opts), emptyResult)
	if obj == nil {
		return emptyResult, err
	}
	return obj.(*v1alpha1.HyperNode), err
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *FakeHyperNodes) UpdateStatus(ctx context.Context, hyperNode *v1alpha1.HyperNode, opts v1.UpdateOptions) (result *v1alpha1.HyperNode, err error) {
	emptyResult := &v1alpha1.HyperNode{}
	obj, err := c.Fake.
		Invokes(testing.NewRootUpdateSubresourceActionWithOptions(hypernodesResource, "status", hyperNode, opts), emptyResult)
	if obj == nil {
		return emptyResult, err
	}
	return obj.(*v1alpha1.HyperNode), err
}

// Delete takes name of the hyperNode and deletes it. Returns an error if one occurs.
func (c *FakeHyperNodes) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	_, err := c.Fake.
		Invokes(testing.NewRootDeleteActionWithOptions(hypernodesResource, name, opts), &v1alpha1.HyperNode{})
	return err
}

// DeleteCollection deletes a collection of objects.
func (c *FakeHyperNodes) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	action := testing.NewRootDeleteCollectionActionWithOptions(hypernodesResource, opts, listOpts)

	_, err := c.Fake.Invokes(action, &v1alpha1.HyperNodeList{})
	return err
}

// Patch applies the patch and returns the patched hyperNode.
func (c *FakeHyperNodes) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.HyperNode, err error) {
	emptyResult := &v1alpha1.HyperNode{}
	obj, err := c.Fake.
		Invokes(testing.NewRootPatchSubresourceActionWithOptions(hypernodesResource, name, pt, data, opts, subresources...), emptyResult)
	if obj == nil {
		return emptyResult, err
	}
	return obj.(*v1alpha1.HyperNode), err
}

// Apply takes the given apply declarative configuration, applies it and returns the applied hyperNode.
func (c *FakeHyperNodes) Apply(ctx context.Context, hyperNode *topologyv1alpha1.HyperNodeApplyConfiguration, opts v1.ApplyOptions) (result *v1alpha1.HyperNode, err error) {
	if hyperNode == nil {
		return nil, fmt.Errorf("hyperNode provided to Apply must not be nil")
	}
	data, err := json.Marshal(hyperNode)
	if err != nil {
		return nil, err
	}
	name := hyperNode.Name
	if name == nil {
		return nil, fmt.Errorf("hyperNode.Name must be provided to Apply")
	}
	emptyResult := &v1alpha1.HyperNode{}
	obj, err := c.Fake.
		Invokes(testing.NewRootPatchSubresourceActionWithOptions(hypernodesResource, *name, types.ApplyPatchType, data, opts.ToPatchOptions()), emptyResult)
	if obj == nil {
		return emptyResult, err
	}
	return obj.(*v1alpha1.HyperNode), err
}

// ApplyStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating ApplyStatus().
func (c *FakeHyperNodes) ApplyStatus(ctx context.Context, hyperNode *topologyv1alpha1.HyperNodeApplyConfiguration, opts v1.ApplyOptions) (result *v1alpha1.HyperNode, err error) {
	if hyperNode == nil {
		return nil, fmt.Errorf("hyperNode provided to Apply must not be nil")
	}
	data, err := json.Marshal(hyperNode)
	if err != nil {
		return nil, err
	}
	name := hyperNode.Name
	if name == nil {
		return nil, fmt.Errorf("hyperNode.Name must be provided to Apply")
	}
	emptyResult := &v1alpha1.HyperNode{}
	obj, err := c.Fake.
		Invokes(testing.NewRootPatchSubresourceActionWithOptions(hypernodesResource, *name, types.ApplyPatchType, data, opts.ToPatchOptions(), "status"), emptyResult)
	if obj == nil {
		return emptyResult, err
	}
	return obj.(*v1alpha1.HyperNode), err
}
